#
# box_and_whisker_annual_plots_with_credible_intervals.R
#
#' Box-and-whisker plots of observed annual target data on the state of the ecosystem with equivalent properties derived from a Monte Carlo StrathE2E run
#'
#' Creates a multi-panel plot comparing a range of observational data on the state of the ecosystem with the distribution of credible values
#' of equivalent properties derived from the final year of a model runs generated by the Monte_Carlo_StrathE2E() function
#'
#' For details of how the distribution of credible output values from StrathE2E are calculated see ?Monte_Carlo_StrathE2E.
#'
#' The function plots a multi-panel page of box-and-whisker plots showing the medians and variability ranges (quartiles as box-and-whisker) of a range of observational data on properties of an ecosystem
#' (shown in black), alongside comparable box-and-whisker plots (shown in red) of equivalent measures derived from the final years of an ensemble of model runs generted by a Monte Carlo methodology (Monte_Carlo_StrathE2E() function)
#'
#' Individual panels of the plot represent groups of similar or related ecosystem properties - annual productions, annual fishery landings, annual P:B ratios, annual food consumtion and diet compositions, nutrient concentrations, and inshore : offshore abundanace ratios.
#'
#' The observational data to be plotted are loaded from the folder Modelname/Variantname/Target_data/annual_target_data-*.csv as part of a read_model() function call and are built into the R-list object generated by read_model().
#' Column 3 of annual_target_data_* (header "Use1_0") is a flag to set whether any given row is used in calculating the likelihood of the observed data given the model setup. Un-used rows of data are omitted from the box and whisker plotting panels
#'
#' Optionally the function can read an example data set for one of the two North Sea model variants supplied with the package.
#'
#' The corresponding measures derived from the final year of a model run generated by the Monte_Carlo_StrathE2E() function call are located in /results/Modelname/Variantname/CredInt_processed_targetresults-*.csv
#'
#' @param model R-list object defining the model configuration compiled by the read_model() function
#' @param use.example (TRUE or FALSE) Option to use pre-computed example data from the internal North Sea model rather than user-generated data (default=FALSE)
#'
#' @return Graphical display in a new graphics window
#'
#' @seealso \code{\link{read_model}}, \code{\link{Monte_Carlo_StrathE2E}}, \code{\link{box_and_whisker_annual_plots}} , \code{\link{box_and_whisker_monthly_plots_with_credible_intervals}}
#'
#' @export
#'
#' @examples
#' # Load the 1970-1999 version of the North Sea model supplied with the package:
#' model <- read_model("North_Sea", "1970-1999")
#'
#' # Either run the Monte_Carlo_StrathE2E() to generate some results, or use the internal example data
#'
#' # Plot the final year comparison of observed and modelled ecosystem state on screen:
#' box_and_whisker_annual_plots_with_credible_intervals(model, use.example=TRUE)
#'
#' #Direct the graphics output to a file (Windows OS)... 
#' pdf("C:/Users/username/Documents/Foldername/plot.pdf",width=8,height=6)      # or jpeg("plot.jpg"), png("plot.png")
#' 	box_and_whisker_annual_plots_with_credible_intervals(model, use.example=TRUE)
#' dev.off()
#
# ---------------------------------------------------------------------
# |                                                                   |
# | Authors: Mike Heath, Ian Thurlbeck                                |
# | Department of Mathematics and Statistics                          |
# | University of Strathclyde, Glasgow                                |
# |                                                                   |
# | Date of this version: October 2019                                |
# |                                                                   |
# ---------------------------------------------------------------------

box_and_whisker_annual_plots_with_credible_intervals <- function(model, use.example=FALSE) {

start_par = par()$mfrow
on.exit(par(mfrow = start_par))

	resultsdir		<- elt(model, "setup", "resultsdir")
	model.ident		<- elt(model, "setup", "model.ident")
	model.path		<- elt(model, "setup", "model.path")
	model.name 		<- elt(model, "setup", "model.name")
	model.variant 		<- elt(model, "setup", "model.variant")

	annualtargetdata	<- read_annual_target_data(model.path)

corefilename<-"CredInt_processed_targetresults"

if(use.example==TRUE){
	cfilename<-paste(CREDINT_DIR,"/",corefilename,sep="")
	credfile <- examplecsvname(model.name, model.variant, cfilename)
	cat(paste("Reading internally stored example results for the ",model.name," ",model.variant," model","\n",sep=""))
}

if(use.example==FALSE){
	credpath	<- makepath(resultsdir, CREDINT_DIR)
	credfile	<- csvname(credpath, corefilename, model.ident)
	cat("Reading credible interval processed data from '", credfile, "'\n", sep="")
}

	if (! file.exists(credfile)) {
		cat("Error: cannot find credible interval output file: ", credfile, "\n")
		stop("Please run the Monte Carlo function!\n")
	}

	credintervaldata	<- readcsv(credfile, row.names=1)	# first column is row names

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	#Function to over-plot the modelci data as box and whiskers
	plot_ci_data<-function(yaxmin,yaxmax,var.list,lgplot=TRUE){
   		modeldata2plot<-(credintervaldata[2:6,var.list[1]])

		#Trap cases where the lower extreme or quartile is zero and the median is positive, and reset these to the yaxis minimum
		if(credintervaldata[2,var.list[1]]==0 & credintervaldata[4,var.list[1]]>0) modeldata2plot[1]<-yaxmin*0.1
		if(credintervaldata[3,var.list[1]]==0 & credintervaldata[4,var.list[1]]>0) modeldata2plot[2]<-yaxmin*0.1

   		for(jj in 2:length(var.list)) {
			tmpmodeldata2plot<-credintervaldata[2:6,var.list[jj]]

			#Trap cases where the lower extreme or quartile is zero and the median is positive, and reset these to the yaxis minimum
			if(credintervaldata[2,var.list[jj]]==0 & credintervaldata[4,var.list[jj]]>0) tmpmodeldata2plot[1]<-yaxmin*0.1
			if(credintervaldata[3,var.list[jj]]==0 & credintervaldata[4,var.list[jj]]>0) tmpmodeldata2plot[2]<-yaxmin*0.1

			modeldata2plot <- c(modeldata2plot,tmpmodeldata2plot)

		}

   		if(length(modeldata2plot[which(modeldata2plot<0)]>0)){
   			modeldata2plot[which(modeldata2plot<0)]<-(yaxmin*0.1)
		}
   		if(lgplot==TRUE) modeldata2plot<-log10(modeldata2plot)
   		array2plot<- array(dim=c(5,length(var.list)),modeldata2plot)
   		bxpdata<-list(stats=array2plot,n=rep(100,length(var.list)),conf=NULL,out=numeric(length=0))
   		bxp(bxpdata,add=TRUE,boxwex=0.3,at=seq(1,length(var.list))+0.35,horizontal=TRUE,yaxt="n",xaxt="n",boxcol="red",whiskcol="red",whisklty="solid",medcol="red",staplecol="red")
	}


	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


	sim_targetdata<-annualtargetdata[,1:2]
	names(sim_targetdata)<-c("observed","observed_sd")

	annualtargetnames<-rep("xx",nrow(annualtargetdata))

	annualtargetnames[which(annualtargetdata[,4]=="Obs_TAPP")] <- "Total phyt."
	annualtargetnames[which(annualtargetdata[,4]=="Obs_NP")] <- "New primary phyt."
	annualtargetnames[which(annualtargetdata[,4]=="Obs_KelpP")] <- "Macrophyte carbon"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_OmnizooP")] <- "Omniv.zooplankton"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_CarnzooP")] <- "Carniv.zooplankton"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_PFishP")] <- "Planktiv.fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_DFishP")] <- "Demersal fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_BensuspP")] <- "Susp/dep.benthos"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_BencarnP")] <- "Carn/scav.benthos"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_birdP")] <- "Seabirds"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_sealP")] <- "Pinnipeds"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_cetaP")] <- "Ceataceans"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_maxbenthslar")] <- "Susp/dep.benthos larv"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_maxbenthclar")] <- "Carn/scav.benthos larv"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Conpfishfish")] <- "Plank.fish by fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Condfishfish")] <- "Dem.fish by fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Conzoofish")] <- "Zooplankton by fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Conzoocarnz")] <- "Omniv.zoo by carniv.zoo."
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Conbenfish")] <- "Benthos by fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Contotal_bird")] <- "Total by birds"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Proppfishbird")] <- "Plank.fish in bird diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Propdfishbird")] <- "Dem.fish in bird diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Propmfishbird")] <- "Mig.fish in bird diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Propdiscbird")] <- "Disc. in bird diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Contotal_seal")] <- "Total by pinnipeds"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Proppfishseal")] <- "Plank.fish in pinn. diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Propdfishseal")] <- "Dem.fish in pinn. diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Propmfishseal")] <- "Mig.fish in pinn. diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Contotal_ceta")] <- "Total by cetaceans"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Proppfishceta")] <- "Plank.fish in cet. diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Propdfishceta")] <- "Dem.fish in cet. diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Propmfishceta")] <- "Mig.fish in cet. diet"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Pland_livewt")] <- "Plank.fish landings"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Dland_livewt")] <- "Dem.fish landings"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Mland_livewt")] <- "Mig.fish landings"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Bsland_livewt")] <- "Susp/dep.benthos landings" 
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Bcland_livewt")] <- "Carn/scav.benthos landings"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Zcland_livewt")] <- "Pel.invert. landings"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Kland_livewt")] <- "Macrop. harvest"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_kelp_pb")] <- "Macrop. P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_benslar_pb")] <- "Susp/dep.benthos larv. P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_benclar_pb")] <- "Carn/scav.benthos larv. P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_bens_pb")] <- "Susp/dep.benthos P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_benc_pb")] <- "Carn/scav.benthos P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_herb_pb")] <- "Omniv.zooplankton P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_carn_pb")] <- "Carniv.zooplankton P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_fishplar_pb")] <- "Plank.fish larvae P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_fishdlar_pb")] <- "Dem.fish larvae P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_fishp_pb")] <- "Plank.fish P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_fishd_pb")] <- "Dem.fish P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_fishm_pb")] <- "Mig.fish P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_bird_pb")] <- "Bird P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_seal_pb")] <- "Pinniped P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_ceta_pb")] <- "Cetacean P/B"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_exud_C_kelp")] <- "Prop. macrop. prod. exuded"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_kelp_NC")] <- "Macrop. N/C ratio"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Denitrif")] <- "Denitrification"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Dfdiscardp")] <- "Dem.fish discard/catch"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_s_x_ammonia")] <- "Sand porewater ammonia"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_d_x_ammonia")] <- "Mud porewater ammonia"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_s_x_nitrate")] <- "Sand porewater nitrate"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_d_x_nitrate")] <- "Mud porewater nitrate"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_s_x_TON")] <- "Sand %TON"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_d_x_TON")] <- "Mud %TON"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_NDJF_s_nitrate")] <- "Winter surf.nitrate"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_MJJA_s_nitrate")] <- "Summer surf.nitrate"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_NDJF_d_nitrate")] <- "Winter deep nitrate"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_MJJA_d_nitrate")] <- "Summer deep nitrate"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_NDJF_s_ammonia")] <- "Winter surf.ammonia"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_MJJA_s_ammonia")] <- "Summer surf.ammonia"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_NDJF_d_ammonia")] <- "Winter deep ammonia"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_MJJA_d_ammonia")] <- "Summer deep ammonia"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_carn_io_ratio")] <- "Carniv.zooplanton"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_herb_io_ratio")] <- "Omniv.zooplankton"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_phyt_io_ratio")] <- "Surf.phytoplankton"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_nit_io_ratio")] <- "Surf.nitrate"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_amm_io_ratio")] <- "Surf.ammonia"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_pfish_io_ratio")] <- "Plank.fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_dfish_io_ratio")] <- "Dem.fish"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_birddisc")] <- "Bird by-catch"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_sealdisc")] <- "Pinniped by-catch"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_cetadisc")] <- "Cetacean by-catch"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_kelp_beachcast")] <- "Macrop. beach-cast"
	annualtargetnames[which(annualtargetdata[,4]=="Obs_Ctland_livewt")] <- "Cetacean landings"

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


	#Read in the model run results output file

	#modeldata<-read.csv(paste(oudir,"model_target_annualresults",AAA,".csv",sep=""),header=TRUE)

	sim_targetdata$model<-0
	for(jjjj in 1:nrow(sim_targetdata)){
		sim_targetdata$model[jjjj]<-credintervaldata[1,jjjj]
	}


	nmeasures<-nrow(sim_targetdata)
	ntargobs<-1000

	annualtarget<-array(0,dim=c(ntargobs,nmeasures))
	#dimensions observations,parameter 
	for(kkk in 1:ntargobs){
		rand<-rnorm(nmeasures,0,1)
		annualtarget[kkk,1:nmeasures]<-sim_targetdata[,1]+(rand*sim_targetdata[,2])
	}
	colnames(annualtarget)<-annualtargetnames
	#~~~~~~~~~~~~~


	#IDs for Annual production rates
	set2plot1.1<-c(
		which(annualtargetdata[,4]=="Obs_KelpP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_TAPP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_NP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Denitrif" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_OmnizooP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_CarnzooP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_BensusP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_BencarnP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_PFishP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_DFishP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_birdP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_sealP" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_cetaP" & annualtargetdata[,3]==1)
	)

	#IDs for Annual fishery landings and by-catch (live weights)
	set2plot1.2<-c(
		which(annualtargetdata[,4]=="Obs_Pland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Dland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Mland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Bsland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Bcland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Zcland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Ctland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Kland_livewt" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_cetadisc" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_sealdisc" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_birddisc" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Dfdiscardp" & annualtargetdata[,3]==1)
	)

	#IDs for annual consumption rates
	set2plot2<-c(
		which(annualtargetdata[,4]=="Obs_Conzoocarnz" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Conzoofish" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Conbenfish" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Conpfishfish" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Condfishfish" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Contotal_bird" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Contotal_seal" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Contotal_ceta" & annualtargetdata[,3]==1),

		which(annualtargetdata[,4]=="Obs_Proppfishbird" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Propdfishbird" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Propmfishbird" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Propdiscbird" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Proppfishseal" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Propdfishseal" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Propmfishseal" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Proppfishceta" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Propdfishceta" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_Propmfishceta" & annualtargetdata[,3]==1)
	)

	#Annual PB and other ratios
	set2plot3<-c(
		which(annualtargetdata[,4]=="Obs_kelp_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_kelp_NC" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_exud_C_kelp" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_kelp_beachcast" & annualtargetdata[,3]==1),

		which(annualtargetdata[,4]=="Obs_herb_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_benslar_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_benclar_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_fishplar_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_fishdlar_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_carn_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_bens_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_benc_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_fishp_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_fishd_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_fishm_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_bird_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_seal_pb" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_ceta_pb" & annualtargetdata[,3]==1)
	)

	#Average nutrient concentaryions in water and sediments
	set2plot4<-c(
		which(annualtargetdata[,4]=="Obs_s_x_ammonia" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_d_x_ammonia" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_s_x_nitrate" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_d_x_nitrate" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_s_x_TON" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_d_x_TON" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_NDJF_s_nitrate" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_MJJA_s_nitrate" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_NDJF_d_nitrate" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_MJJA_d_nitrate" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_NDJF_s_ammonia" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_MJJA_s_ammonia" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_NDJF_d_ammonia" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_MJJA_d_ammonia" & annualtargetdata[,3]==1)
	)

	#Inshore:offshore ratios
	set2plot5<-c(
		which(annualtargetdata[,4]=="Obs_amm_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_nit_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_phyt_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_herb_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_carn_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_pfish_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_dfish_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_bird_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_seal_io_ratio" & annualtargetdata[,3]==1),
		which(annualtargetdata[,4]=="Obs_ceta_io_ratio" & annualtargetdata[,3]==1)
	)


	par(mfrow=c(3,2))
	par(mar=c(4,12,0.5,2.75))

	#tfnu<-0.8 # scaling for x-axis labels
        #tfntsmall<-0.75
	#tfntlarge<-1.1
	#tfntcut<-8

	tfnu<-0.75 # scaling for x-axis labels
        tfntsmall<-1.0
	tfntlarge<-1.0
	tfntcut<-8

	if(length(set2plot1.1)>0){
		#Set y axis limits
			yaxmin<-0.0001
			yaxmax<-2000
			if(length(set2plot1.1)<(tfntcut+1)) tfnt<-tfntlarge # scaling for y-axis labels
			if(length(set2plot1.1)>(tfntcut)) tfnt<-tfntsmall # scaling for y-axis labels
			axlabel<-expression(paste("Production Log"[{10}]," mMN.m"^{-2},".y"^{-1},sep=""))

	#Catch values which are negative and set them to NA
	for(qqqqq in 1:length(set2plot1.1)){
		if(length(which(annualtarget[,set2plot1.1[qqqqq]]<0)>0)){
			annualtarget[which(annualtarget[,set2plot1.1[qqqqq]]<0),set2plot1.1[qqqqq]] <- (NA)
	}}
	boxplot(as.data.frame(log10(annualtarget[,set2plot1.1])),range=0,boxwex=0.3,ylim=c(log10(yaxmin),log10(yaxmax)),horizontal=TRUE,las=1,par(lty=1,cex=0.6,cex.lab=tfnt,cex.axis=tfnt))
	mtext(axlabel,cex=tfnu,side=1,line=2.5)
	plot_ci_data(yaxmin,yaxmax,set2plot1.1,lgplot=TRUE)
	}

	if(length(set2plot1.2)>0){
		#Set y axis limits
			yaxmin<-0.0000001
			yaxmax<-100
			if(length(set2plot1.2)<(tfntcut+1)) tfnt<-tfntlarge # scaling for y-axis labels
			if(length(set2plot1.2)>(tfntcut)) tfnt<-tfntsmall # scaling for y-axis labels
			axlabel<-expression(paste("Landings Log"[{10}]," mMN.m"^{-2},".y"^{-1},sep=""))

	#Catch values which are negative and set them to NA
	for(qqqqq in 1:length(set2plot1.2)){
	if(length(which(annualtarget[,set2plot1.2[qqqqq]]<0)>0)){
	annualtarget[which(annualtarget[,set2plot1.2[qqqqq]]<0),set2plot1.2[qqqqq]] <- (NA)
	}}
	boxplot(as.data.frame(log10(annualtarget[,set2plot1.2])),range=0,boxwex=0.3,ylim=c(log10(yaxmin),log10(yaxmax)),horizontal=TRUE,las=1,par(lty=1,cex=0.6,cex.lab=tfnt,cex.axis=tfnt))
	mtext(axlabel,cex=tfnu,side=1,line=2.5)
	plot_ci_data(yaxmin,yaxmax,set2plot1.2,lgplot=TRUE)
	}



	if(length(set2plot2)>0){
		#Set y axis limits
			yaxmin<-0.00001
			yaxmax<-200
			if(length(set2plot2)<(tfntcut+1)) tfnt<-tfntlarge # scaling for y-axis labels
			if(length(set2plot2)>(tfntcut)) tfnt<-tfntsmall # scaling for y-axis labels
			axlabel<-expression(paste("Consumption Log"[{10}]," mMN.m"^{-2},".y"^{-1},sep=""))

	#Catch values which are negative and set them to NA
	for(qqqqq in 1:length(set2plot2)){
	if(length(which(annualtarget[,set2plot2[qqqqq]]<0)>0)){
	annualtarget[which(annualtarget[,set2plot2[qqqqq]]<0),set2plot2[qqqqq]] <- (NA)
	}}
	boxplot(as.data.frame(log10(annualtarget[,set2plot2])),range=0,boxwex=0.3,ylim=c(log10(yaxmin),log10(yaxmax)),horizontal=TRUE,las=1,par(lty=1,cex=0.6,cex.lab=tfnt,cex.axis=tfnt))
	mtext(axlabel,cex=tfnu,side=1,line=2.5)
	plot_ci_data(yaxmin,yaxmax,set2plot2,lgplot=TRUE)
	}


	if(length(set2plot3)>0){
		#Set y axis limits
			yaxmin<-0.001
			yaxmax<-100
			if(length(set2plot3)<(tfntcut+1)) tfnt<-tfntlarge # scaling for y-axis labels
			if(length(set2plot3)>(tfntcut)) tfnt<-tfntsmall # scaling for y-axis labels
			axlabel<-expression(paste("Log"[{10}]," Annual ratio",sep=""))

	#Catch values which are negative and set them to NA
	for(qqqqq in 1:length(set2plot3)){
	if(length(which(annualtarget[,set2plot3[qqqqq]]<0)>0)){
	annualtarget[which(annualtarget[,set2plot3[qqqqq]]<0),set2plot3[qqqqq]] <- (NA)
	}}
	boxplot(as.data.frame(log10(annualtarget[,set2plot3])),range=0,boxwex=0.3,ylim=c(log10(yaxmin),log10(yaxmax)),horizontal=TRUE,las=1,par(lty=1,cex=0.6,cex.lab=tfnt,cex.axis=tfnt))
	mtext(axlabel,cex=tfnu,side=1,line=2.5)
	plot_ci_data(yaxmin,yaxmax,set2plot3,lgplot=TRUE)
	}

	if(length(set2plot4)>0){
		#Set y axis limits
			yaxmin<-0.0001
			yaxmax<-1000
			if(length(set2plot4)<(tfntcut+1)) tfnt<-tfntlarge # scaling for y-axis labels
			if(length(set2plot4)>(tfntcut)) tfnt<-tfntsmall # scaling for y-axis labels
			axlabel<-expression(paste("Conc. Log"[{10}]," mMN.m"^{-3}," or % by weight",sep=""))

	#Catch values which are negative and set them to NA
	for(qqqqq in 1:length(set2plot4)){
	if(length(which(annualtarget[,set2plot4[qqqqq]]<0)>0)){
	annualtarget[which(annualtarget[,set2plot4[qqqqq]]<0),set2plot4[qqqqq]] <- (NA)
	}}
	boxplot(as.data.frame(log10(annualtarget[,set2plot4])),range=0,boxwex=0.3,ylim=c(log10(yaxmin),log10(yaxmax)),horizontal=TRUE,las=1,par(lty=1,cex=0.6,cex.lab=tfnt,cex.axis=tfnt))
	mtext(axlabel,cex=tfnu,side=1,line=2.5)
	plot_ci_data(yaxmin,yaxmax,set2plot4,lgplot=TRUE)
	}

	if(length(set2plot5)>0){
		#Set y axis limits
			yaxmin<-0
			yaxmax<-8
			if(length(set2plot5)<(tfntcut+1)) tfnt<-tfntlarge # scaling for y-axis labels
			if(length(set2plot5)>(tfntcut)) tfnt<-tfntsmall # scaling for y-axis labels
			axlabel<-expression(paste("Inshore:offshore ratios"))

	#Catch values which are negative and set them to NA
	for(qqqqq in 1:length(set2plot5)){
	if(length(which(annualtarget[,set2plot5[qqqqq]]<0)>0)){
	annualtarget[which(annualtarget[,set2plot5[qqqqq]]<0),set2plot5[qqqqq]] <- (NA)
	}}
	boxplot(as.data.frame((annualtarget[,set2plot5])),range=0,boxwex=0.3,ylim=c(yaxmin,yaxmax),horizontal=TRUE,las=1,par(lty=1,cex=0.6,cex.lab=tfnt,cex.axis=tfnt))
	mtext(axlabel,cex=tfnu,side=1,line=2.5)
	plot_ci_data(yaxmin,yaxmax,set2plot5,lgplot=FALSE)
	}

}

